@@ -415,8 +415,8 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 }
                 break;
             case KEY_POPUP_MENU:
-                show_window_menu (c, frameX (c) + frameLeft (c),
-                                     frameY (c) + frameTop (c),
+                show_window_menu (c, frameExtentX (c) + frameLeft (c),
+                                     frameExtentY (c) + frameTop (c),
                                      Button1, ev->time);
                 break;
             case KEY_FILL_WINDOW:
@@ -1960,6 +1960,12 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
             clientSetFullscreenMonitor (c, (gint) ev->data.l[0], (gint) ev->data.l[1],
                                            (gint) ev->data.l[2], (gint) ev->data.l[3]);
         }
+        else if ((ev->message_type == display_info->atoms[GTK_SHOW_WINDOW_MENU]) && (ev->format == 32))
+        {
+            TRACE ("client \"%s\" (0x%lx) has received a GTK_SHOW_WINDOW_MENU event", c->name, c->window);
+            show_window_menu (c, (gint) ev->data.l[1], (gint) ev->data.l[2], Button3, (Time) myDisplayGetCurrentTime (display_info));
+        }
+
     }
     else
     {
@@ -2407,14 +2413,14 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
     gboolean is_transient;
     gint x, y;
 
-    TRACE ("entering show_window_menu");
+    TRACE ("entering show_window_menu (%d,%d)", px, py);
 
     if ((button != Button1) && (button != Button3))
     {
         return;
     }
 
-    if (!c || !FLAG_TEST_ALL (c->xfwm_flags, XFWM_FLAG_HAS_MENU | XFWM_FLAG_VISIBLE))
+    if (!c || !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
     {
         return;
     }
@@ -2428,7 +2434,10 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
 
     c->button_status[MENU_BUTTON] = BUTTON_STATE_PRESSED;
     frameQueueDraw (c, FALSE);
-    y = (gdouble) c->y;
+    if (CLIENT_HAS_FRAME (c))
+    {
+        y = c->y;
+    }
     ops = MENU_OP_DELETE | MENU_OP_MINIMIZE_ALL | MENU_OP_WORKSPACES | MENU_OP_MOVE | MENU_OP_RESIZE;
     insensitive = 0;
 
@@ -2473,6 +2482,12 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
         ops |= MENU_OP_STICK;
     }
 
+    if (!FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER))
+    {
+        insensitive |= MENU_OP_SHADE | MENU_OP_UNSHADE;
+    }
+
+
     if (!FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_CLOSE))
     {
         insensitive |= MENU_OP_DELETE;
