@@ -58,7 +58,6 @@ struct _ClientPair
 static Client *client_focus  = NULL;
 static Client *pending_focus = NULL;
 static Client *user_focus    = NULL;
-static Client *last_ungrab   = NULL;
 static Client *delayed_focus = NULL;
 static guint focus_timeout   = 0;
 
@@ -176,11 +175,16 @@ clientFocusNew(Client * c)
         }
         else if (client_focus->win_layer < c->win_layer)
         {
-            /* We short don't use focus stealing prevention against upper layers */
+            /* We don't use focus stealing prevention against upper layers */
             TRACE ("Ignoring startup prevention because the current focused window is on a lower layer");
             give_focus = TRUE;
             prevented = FALSE;
         }
+        else if (FLAG_TEST (client_focus->xfwm_flags, XFWM_FLAG_MOVING_RESIZING))
+        {
+            give_focus = FALSE;
+            prevented = TRUE;
+        }
         else if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME | CLIENT_FLAG_HAS_USER_TIME))
         {
             TRACE ("Current time is %u, time for \"%s\" is %u",
@@ -193,11 +197,13 @@ clientFocusNew(Client * c)
             }
         }
     }
-
-    if ((give_focus) || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
+    if (FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
         give_focus = TRUE;
-        if ((client_focus) && !(clientIsTransientOrModalFor (c, client_focus)))
+    }
+    if (give_focus)
+    {
+        if (client_focus)
         {
             clientAdjustFullscreenLayer (client_focus, FALSE);
         }
@@ -206,28 +212,26 @@ clientFocusNew(Client * c)
         clientSetFocus (screen_info, c,
                         myDisplayGetCurrentTime (display_info),
                         FOCUS_IGNORE_MODAL);
-        clientPassGrabMouseButton (c);
     }
     else
     {
         Client *c2 = clientGetFocus();
 
-        /*
-         * Place windows under the currently focused only if focus
-         * stealing prevention had prevented the focus transition,
-         * otherwise, leave the unfocused window on top.
-         */
+        clientSortRing(c);
         if ((c2 != NULL) && (c2->win_layer == c->win_layer) && prevented)
         {
-            clientSortRing(c);
+            /*
+             * Place windows under the currently focused only if focus
+             * stealing prevention had prevented the focus transition,
+             * otherwise, leave the unfocused window on top.
+             */
             clientLower (c, c2->frame);
-            clientSortRing(c2);
         }
         else
         {
             clientRaise (c, None);
-            clientSortRing(c);
         }
+        clientSortRing(c2);
 
         if (prevented)
         {
@@ -236,7 +240,6 @@ clientFocusNew(Client * c)
         }
 
         clientShow (c, TRUE);
-        clientGrabMouseButton (c);
         clientSetNetState (c);
     }
 
@@ -359,11 +362,6 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, GList *exclude_list)
         return;
     }
 
-    if (current_focus == last_ungrab)
-    {
-        clientPassGrabMouseButton (NULL);
-    }
-
     display_info = screen_info->display_info;
     top_most = clientGetTopMostFocusable (screen_info, look_in_layer, exclude_list);
 
@@ -392,11 +390,6 @@ clientAcceptFocus (Client * c)
     {
         return TRUE;
     }
-    /* First check GNOME protocol */
-    if (c->win_hints & WIN_HINTS_SKIP_FOCUS)
-    {
-        return FALSE;
-    }
     if ((c->screen_info->params->focus_hint)
         && !FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
     {
@@ -411,9 +404,11 @@ clientSortRing(Client *c)
 {
     ScreenInfo *screen_info;
 
-    g_return_if_fail (c != NULL);
     TRACE ("entering clientSortRing");
-
+    if (c == NULL)
+    {
+        return;
+    }
     screen_info = c->screen_info;
     if ((screen_info->client_count > 2) && (c != screen_info->clients))
     {
@@ -467,6 +462,7 @@ clientFocusNone (ScreenInfo *screen_info, Client *previous, guint32 timestamp)
 
     if (previous)
     {
+        clientSetNetState (previous);
         frameQueueDraw (previous, FALSE);
         if (previous->screen_info != screen_info)
         {
@@ -499,16 +495,14 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         return;
     }
 
-    /*
-       We can release the button mouse grab if we don't raise on click or if the focused window
-       is the one that has been raised at last.
-     */
-    if (!(screen_info->params->raise_on_click) || (c == clientGetLastRaise (screen_info)))
-    {
-        clientPassGrabMouseButton (c);
-    }
-
     client_focus = c;
+    if (c2)
+    {
+        clientSetNetState (c2);
+        clientAdjustFullscreenLayer (c2, FALSE);
+        frameQueueDraw (c2, FALSE);
+        clientUpdateOpacity (c2);
+    }
     if (c)
     {
         user_focus = c;
@@ -521,23 +515,13 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         {
             TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window);
             FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-            clientSetNetState (c);
         }
+        clientSetNetState (c);
         clientAdjustFullscreenLayer (c, TRUE);
         frameQueueDraw (c, FALSE);
         clientUpdateOpacity (c);
     }
-    if (c2)
-    {
-        if (c)
-        {
-            clientAdjustFullscreenLayer (c2, FALSE);
-        }
-        frameQueueDraw (c2, FALSE);
-        clientUpdateOpacity (c2);
-    }
     clientSetNetActiveWindow (screen_info, c, 0);
-    clientClearDelayedFocus ();
     pending_focus = NULL;
 }
 
@@ -742,7 +726,6 @@ clientGrabMouseButtonForAll (ScreenInfo *screen_info)
     {
         clientGrabMouseButton (c);
     }
-    clientClearLastUngrab ();
 }
 
 void
@@ -758,62 +741,19 @@ clientUngrabMouseButtonForAll (ScreenInfo *screen_info)
     {
         clientUngrabMouseButton (c);
     }
-    clientClearLastUngrab ();
-}
-
-void
-clientPassGrabMouseButton (Client * c)
-{
-    TRACE ("entering clientPassMouseGrabButton");
-
-    if (c == NULL)
-    {
-        if (last_ungrab)
-        {
-            clientGrabMouseButton (last_ungrab);
-        }
-        last_ungrab = NULL;
-        return;
-    }
-
-    if (last_ungrab == c)
-    {
-        return;
-    }
-
-    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
-    if (last_ungrab)
-    {
-        clientGrabMouseButton (last_ungrab);
-    }
-    clientUngrabMouseButton (c);
-    last_ungrab = c;
-}
-
-Client *
-clientGetLastUngrab (void)
-{
-    return last_ungrab;
-}
-
-void
-clientClearLastUngrab (void)
-{
-    last_ungrab = NULL;
 }
 
 static gboolean
 delayed_focus_cb (gpointer data)
 {
     ScreenInfo *screen_info;
-    DisplayInfo *display_info;
+    guint32 timestamp = (guint32) GPOINTER_TO_INT (data);
 
     TRACE ("entering delayed_focus_cb");
     g_return_val_if_fail (delayed_focus != NULL, FALSE);
 
     screen_info = delayed_focus->screen_info;
-    display_info = screen_info->display_info;
-    clientSetFocus (screen_info, delayed_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+    clientSetFocus (screen_info, delayed_focus, timestamp, NO_FOCUS_FLAG);
     focus_timeout = 0;
     delayed_focus = NULL;
 
@@ -823,7 +763,7 @@ delayed_focus_cb (gpointer data)
 void
 clientClearDelayedFocus (void)
 {
-    if(focus_timeout)
+    if (focus_timeout)
     {
         g_source_remove (focus_timeout);
         focus_timeout = 0;
@@ -832,7 +772,7 @@ clientClearDelayedFocus (void)
 }
 
 void
-clientAddDelayedFocus (Client *c)
+clientAddDelayedFocus (Client *c, guint32 timestamp)
 {
     ScreenInfo *screen_info;
 
@@ -841,7 +781,7 @@ clientAddDelayedFocus (Client *c)
     focus_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
                                         screen_info->params->focus_delay,
                                         (GSourceFunc) delayed_focus_cb,
-                                        NULL, NULL);
+                                        GINT_TO_POINTER (timestamp), NULL);
 }
 
 Client *
