@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2011 Olivier Fourdan,
+        xfwm4    - (c) 2002-2015 Olivier Fourdan,
                        2008 Jannis Pohlmann
 
  */
@@ -46,7 +46,10 @@
 
 #define CHANNEL_XFWM            "xfwm4"
 #define THEMERC                 "themerc"
-#define XPM_COLOR_SYMBOL_SIZE   22
+#define XPM_COLOR_SYMBOL_SIZE   24
+
+#define KEYMAP_UPDATE_TIMEOUT   250 /* ms */
+static guint keymap_timeout   = 0;
 
 /* Forward static decls. */
 
@@ -309,6 +312,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     static const char *side_names[] = {
         "left",
         "right",
+        "top", /* ** unused ** */
         "bottom"
     };
 
@@ -350,6 +354,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
         "light",
         "dark",
         "mid",
+        "fg",
         "bg",
         "light",
         "dark",
@@ -358,6 +363,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
         "light",
         "dark",
         "mid",
+        "fg",
         "bg",
         "light",
         "dark",
@@ -380,6 +386,8 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
         "normal",
         "normal",
         "normal",
+        "normal",
+        "insensitive",
         "insensitive",
         "insensitive",
         "insensitive",
@@ -408,19 +416,42 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     desc = NULL;
     i = 0;
 
-    while (ui_part[i] && ui_state[i])
+    /* Load gtk theme colors first */
+    for (i = 0; i < XPM_COLOR_SYMBOL_SIZE; i++)
     {
         gchar *color;
 
         color = getUIStyle  (widget, ui_part[i], ui_state[i]);
         setStringValue (rc[i].option, color, rc);
         g_free (color);
-        ++i;
     }
 
+    /* Then load xfwm4 theme values */
     theme = getThemeDir (getStringValue ("theme", rc), THEMERC);
     parseRc (THEMERC, theme, rc);
 
+    /* And finally redo a pass for transitive definitions of colors */
+    for (i = 0; i < XPM_COLOR_SYMBOL_SIZE; i++)
+    {
+        const gchar *value;
+
+        /*
+         * Allow color definition based on another, e.g:
+         * active_text_color=active_text_color_2
+         * active_text_shadow_color=active_hilight_2
+         * etc.
+         */
+        value = g_value_get_string(rc[i].value);
+        for (j = 0; j < XPM_COLOR_SYMBOL_SIZE; j++)
+        {
+            if (!g_ascii_strcasecmp (value, rc[j].option))
+            {
+                setStringValue (rc[i].option, g_value_get_string(rc[j].value), rc);
+                break;
+            }
+        }
+    }
+
     screen_info->params->shadow_delta_x =
         - getIntValue ("shadow_delta_x", rc);
     screen_info->params->shadow_delta_y =
@@ -488,8 +519,11 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     screen_info->white_gc = widget->style->white_gc;
     g_object_ref (G_OBJECT (widget->style->white_gc));
 
-    for (i = 0; i < SIDE_TOP; i++) /* Keep SIDE_TOP for later */
+    for (i = 0; i < SIDE_COUNT; i++)
     {
+        if (i == SIDE_TOP)
+            continue;  /* There is no top decoration per se. */
+
         g_snprintf(imagename, sizeof (imagename), "%s-active", side_names[i]);
         xfwmPixmapLoad (screen_info, &screen_info->sides[i][ACTIVE], theme, imagename, colsym);
 
@@ -615,6 +649,10 @@ loadKeyBindings (ScreenInfo *screen_info)
     parseShortcut (screen_info, KEY_TILE_LEFT, "tile_left_key", shortcuts);
     parseShortcut (screen_info, KEY_TILE_RIGHT, "tile_right_key", shortcuts);
     parseShortcut (screen_info, KEY_TILE_UP, "tile_up_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_DOWN_LEFT, "tile_down_left_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_DOWN_RIGHT, "tile_down_right_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_UP_LEFT, "tile_up_left_key", shortcuts);
+    parseShortcut (screen_info, KEY_TILE_UP_RIGHT, "tile_up_right_key", shortcuts);
     parseShortcut (screen_info, KEY_TOGGLE_ABOVE, "above_key", shortcuts);
     parseShortcut (screen_info, KEY_TOGGLE_FULLSCREEN, "fullscreen_key", shortcuts);
     parseShortcut (screen_info, KEY_UP_WORKSPACE, "up_workspace_key", shortcuts);
@@ -652,6 +690,7 @@ loadSettings (ScreenInfo *screen_info)
         {"active_hilight_1", NULL, G_TYPE_STRING, FALSE},
         {"active_shadow_1", NULL, G_TYPE_STRING, FALSE},
         {"active_mid_1", NULL, G_TYPE_STRING, FALSE},
+        {"active_text_color_2", NULL, G_TYPE_STRING, FALSE},
         {"active_color_2", NULL, G_TYPE_STRING, FALSE},
         {"active_hilight_2", NULL, G_TYPE_STRING, FALSE},
         {"active_shadow_2", NULL, G_TYPE_STRING, FALSE},
@@ -660,6 +699,7 @@ loadSettings (ScreenInfo *screen_info)
         {"inactive_hilight_1", NULL, G_TYPE_STRING, FALSE},
         {"inactive_shadow_1", NULL, G_TYPE_STRING, FALSE},
         {"inactive_mid_1", NULL, G_TYPE_STRING, FALSE},
+        {"inactive_text_color_2", NULL, G_TYPE_STRING, FALSE},
         {"inactive_color_2", NULL, G_TYPE_STRING, FALSE},
         {"inactive_hilight_2", NULL, G_TYPE_STRING, FALSE},
         {"inactive_shadow_2", NULL, G_TYPE_STRING, FALSE},
@@ -667,6 +707,7 @@ loadSettings (ScreenInfo *screen_info)
         /* You can change the order of the following parameters */
         {"activate_action", NULL, G_TYPE_STRING, TRUE},
         {"borderless_maximize", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"titleless_maximize", NULL, G_TYPE_BOOLEAN, TRUE},
         {"box_move", NULL, G_TYPE_BOOLEAN, TRUE},
         {"box_resize", NULL, G_TYPE_BOOLEAN, TRUE},
         {"button_layout", NULL, G_TYPE_STRING, TRUE},
@@ -678,6 +719,7 @@ loadSettings (ScreenInfo *screen_info)
         {"cycle_draw_frame", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_hidden", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_minimum", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"cycle_preview", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_tabwin_mode", NULL, G_TYPE_INT, FALSE},
         {"cycle_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
         {"double_click_time", NULL, G_TYPE_INT, TRUE},
@@ -706,7 +748,6 @@ loadSettings (ScreenInfo *screen_info)
         {"raise_with_any_button", NULL, G_TYPE_BOOLEAN, TRUE},
         {"repeat_urgent_blink", NULL, G_TYPE_BOOLEAN, TRUE},
         {"resize_opacity", NULL, G_TYPE_INT, TRUE},
-        {"restore_on_move", NULL, G_TYPE_BOOLEAN, TRUE},
         {"scroll_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
         {"shadow_delta_height", NULL, G_TYPE_INT, TRUE},
         {"shadow_delta_width", NULL, G_TYPE_INT, TRUE},
@@ -741,6 +782,7 @@ loadSettings (ScreenInfo *screen_info)
         {"wrap_resistance", NULL, G_TYPE_INT, TRUE},
         {"wrap_windows", NULL, G_TYPE_BOOLEAN, TRUE},
         {"wrap_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"zoom_desktop", NULL, G_TYPE_BOOLEAN, TRUE},
         {NULL, NULL, G_TYPE_INVALID, FALSE}
     };
 
@@ -755,6 +797,8 @@ loadSettings (ScreenInfo *screen_info)
 
     screen_info->params->borderless_maximize =
         getBoolValue ("borderless_maximize", rc);
+    screen_info->params->titleless_maximize =
+        getBoolValue ("titleless_maximize", rc);
     screen_info->params->box_resize =
         getBoolValue ("box_resize", rc);
     screen_info->params->box_move =
@@ -769,6 +813,8 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("cycle_draw_frame", rc);
     screen_info->params->cycle_hidden =
         getBoolValue ("cycle_hidden", rc);
+    screen_info->params->cycle_preview =
+        getBoolValue ("cycle_preview", rc);
     screen_info->params->cycle_tabwin_mode =
         CLAMP (getIntValue ("cycle_tabwin_mode", rc), 0, 1);
     screen_info->params->cycle_workspaces =
@@ -795,8 +841,6 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("repeat_urgent_blink", rc);
     screen_info->params->urgent_blink =
         getBoolValue ("urgent_blink", rc);
-    screen_info->params->restore_on_move =
-        getBoolValue ("restore_on_move", rc);
     screen_info->params->frame_opacity =
         CLAMP (getIntValue ("frame_opacity", rc), 0, 100);
     screen_info->params->inactive_opacity =
@@ -839,6 +883,8 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("use_compositing", rc);
     screen_info->params->wrap_workspaces =
         getBoolValue ("wrap_workspaces", rc);
+    screen_info->params->zoom_desktop =
+        getBoolValue ("zoom_desktop", rc);
 
     screen_info->params->wrap_layout =
         getBoolValue ("wrap_layout", rc);
@@ -1262,6 +1308,10 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                     screen_info->params->wrap_workspaces = g_value_get_boolean (value);
                     placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
                 }
+                else if (!strcmp (name, "zoom_desktop"))
+                {
+                    screen_info->params->zoom_desktop = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "wrap_windows"))
                 {
                     screen_info->params->wrap_windows = g_value_get_boolean (value);
@@ -1271,6 +1321,11 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                     screen_info->params->borderless_maximize = g_value_get_boolean (value);
                     reloadScreenSettings (screen_info, UPDATE_MAXIMIZE);
                 }
+                else if (!strcmp (name, "titleless_maximize"))
+                {
+                    screen_info->params->titleless_maximize = g_value_get_boolean (value);
+                    reloadScreenSettings (screen_info, UPDATE_MAXIMIZE);
+                }
                 else if (!strcmp (name, "cycle_minimum"))
                 {
                     screen_info->params->cycle_minimum = g_value_get_boolean (value);
@@ -1291,6 +1346,10 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->cycle_apps_only = g_value_get_boolean (value);
                 }
+                else if (!strcmp (name, "cycle_preview"))
+                {
+                    screen_info->params->cycle_preview = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "focus_hint"))
                 {
                     screen_info->params->focus_hint = g_value_get_boolean (value);
@@ -1308,10 +1367,6 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                     screen_info->params->raise_with_any_button = g_value_get_boolean (value);
                     update_grabs (screen_info);
                 }
-                else if (!strcmp (name, "restore_on_move"))
-                {
-                    screen_info->params->restore_on_move = g_value_get_boolean (value);
-                }
                 else if (!strcmp (name, "scroll_workspaces"))
                 {
                     screen_info->params->scroll_workspaces = g_value_get_boolean (value);
@@ -1389,18 +1444,46 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
     }
 }
 
-static void
-cb_keys_changed (GdkKeymap *keymap, ScreenInfo *screen_info)
+
+static gboolean
+keymap_reload (gpointer data)
 {
+    ScreenInfo *screen_info = (ScreenInfo *) data;
+
+    g_return_val_if_fail (screen_info != NULL, FALSE);
+    TRACE ("entering keymap_reload");
+
     /* Recompute modifiers mask in case it changed */
     initModifiers (myScreenGetXDisplay (screen_info));
 
-    /* Ungrab/regrab shortcuts */
+    /* Ungrab shortcuts and clear existing bindings */
     myScreenUngrabKeys (screen_info);
+    unloadKeyBindings (screen_info);
+
+    /* Reload new key bindings and regrab shortcuts */
+    loadKeyBindings  (screen_info);
     myScreenGrabKeys (screen_info);
 
-    /* Uupdate all grabs for mouse buttons */
+    /* Update all grabs for mouse buttons */
     clientUpdateAllFrames (screen_info, UPDATE_BUTTON_GRABS);
+
+    /* We're done */
+    keymap_timeout = 0;
+
+    return FALSE;
+}
+
+static void
+cb_keys_changed (GdkKeymap *keymap, ScreenInfo *screen_info)
+{
+    if (keymap_timeout)
+    {
+        g_source_remove (keymap_timeout);
+    }
+    keymap_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                         KEYMAP_UPDATE_TIMEOUT,
+                                         (GSourceFunc) keymap_reload,
+                                         (gpointer) screen_info, NULL);
 }
 
 static void
