@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
 
  */
 
@@ -104,6 +104,10 @@ enum
     KEY_TILE_LEFT,
     KEY_TILE_RIGHT,
     KEY_TILE_UP,
+    KEY_TILE_DOWN_LEFT,
+    KEY_TILE_DOWN_RIGHT,
+    KEY_TILE_UP_LEFT,
+    KEY_TILE_UP_RIGHT,
     KEY_TOGGLE_ABOVE,
     KEY_TOGGLE_FULLSCREEN,
     KEY_UP_WORKSPACE,
@@ -198,6 +202,7 @@ struct _XfwmParams
     int title_shadow[2];
     int wrap_resistance;
     gboolean borderless_maximize;
+    gboolean titleless_maximize;
     gboolean box_move;
     gboolean box_resize;
     gboolean click_to_focus;
@@ -205,6 +210,7 @@ struct _XfwmParams
     gboolean cycle_draw_frame;
     gboolean cycle_hidden;
     gboolean cycle_minimum;
+    gboolean cycle_preview;
     gboolean cycle_workspaces;
     gboolean focus_hint;
     gboolean focus_new;
@@ -215,7 +221,6 @@ struct _XfwmParams
     gboolean raise_on_focus;
     gboolean raise_with_any_button;
     gboolean repeat_urgent_blink;
-    gboolean restore_on_move;
     gboolean scroll_workspaces;
     gboolean show_app_icon;
     gboolean show_dock_shadow;
@@ -236,6 +241,7 @@ struct _XfwmParams
     gboolean wrap_layout;
     gboolean wrap_windows;
     gboolean wrap_workspaces;
+    gboolean zoom_desktop;
 };
 
 gboolean                 loadSettings                           (ScreenInfo *);
